// Create-on-demand map of mappings for conferences.
//
// $Id: map-cache.cc,v 1.10 2005/04/02 21:21:48 ceder Exp $
// Copyright (C) 1994  Per Cederqvist and Inge Wallin
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#include <config.h>

#include "kom-types.h"
#include "misc.h"
#include "map-cache.h"
#include "async_dispatcher.h"
#include "async_text_created.h"
#include "text-stat.h"
#include "conference.h"

Map_cache::Map_cache(const Conferences &c, Async_dispatcher *disp)
 : m_conn(c.connection()),
   m_conf_cache(c.read_conf_cache()),
   async_disp(disp),
   mapmap()
{
    Kom_auto_ptr<Async_callback<Async_text_created> > ptr;
    ptr.reset(async_callback_method(this, &Map_cache::new_text).release());
    cr_tag = disp->register_text_created_handler(ptr);
}


Map_cache::~Map_cache()
{
    async_disp->unregister_text_created_handler(cr_tag);
}


Connection*
Map_cache::connection() const
{
    return m_conn;
}

 
Text_mapping
Map_cache::get(Conf_no cno)
{
    Text_mapping &r = mapmap[cno];
    if (!r.valid())
	r = Text_mapping(Conference(cno, m_conf_cache));
    return r;
}

void
Map_cache::new_text(const Async_text_created &msg)
{
    Text_stat t = msg.text_stat();
    for (int ix = 0; ix < t.num_recipients(); ix++)
    {
	Recipient *recpt = t.recipient(ix);
	Text_mapping map = get(recpt->recipient());
	map.add(recpt->local_no(), t.text_no());
    }
}
